Item = {
	_prefix: "i_",
	extract: function(a, data, callback) {
		a = $(a);
		var b = a.attr('rel').match(/\?id=([^&]+)&tab=([^&]+)/);
		if (data) {
			var item = Object.create(this);
			for (var key in data) item[key] = data[key];
			item.init(Number(b[1]), b[2], a.find('img[src]').attr('src'), callback);
			return item;
		}
		return this.create(Number(b[1]), b[2], a.find('img[src]').attr('src'), callback);
	},
	create: function(iId, type, img, callback) {
		return Object.create(this).init(iId, type, img, callback);
	},
	init: function(iId, type, img, callback) {
		this.id = Number(iId);
		this.type = type;
		this.img = img;
		this.lastLoad = 0;
		this.stacks = {};
		this.load(callback);
		return this;
	},
	addStack: function($item) {
		var stack = {
			id: Number($item.children('input[type="hidden"][value]').attr('value')),
			count: Number($item.children('select[name]').children('option[selected]').attr('value')),
		};
		this.stacks[stack.id] = stack;
	},
	join: function(other) {
		for (var sId in other.stacks) this.stacks[sId] = other.stacks[sId];
	},
	loadData: function(callback) {
		var item = this;
		$.ajax({
			url: HOST+"/includes/itemajax.php?id="+this.id+"&tab="+this.type,
		}).done(function(html) {
			var $doc = $((new DOMParser()).parseFromString(html, "text/html"));
			var $base = $doc.find('body').children('div');
			var a;
			
			a = $base.find("img").attr("src");
			if (a) item.rarity = a.match(/\/images\/layout\/tooltip_(\d+)star.png/)[1];
			else item.rarity = null;
			
			a = $base.children("div").children("div");
			item.name = $(a[0]).text();
			item.category = $(a[1]).text();
			item.description = $($base.children("div")[1]).text().trim();
			
			a = html.match(/>Sell Value: (\d+)</);
			if (a) item.value = Number(a[1]);
			a = html.match(/>Food Points: (\d+)</);
			if (a) item.foodPoints = Number(a[1]);
			a = html.match(/>Required Level: (\d+)</);
			if (a) item.requiredLevel = Number(a[1]);
			
			item.lastLoad = Date.now();
			item.save();
			if (callback) callback(item);
		});
	},
	save: function() {
		var data = {};
		var a = Object.getOwnPropertyNames(this);
		for (var i=0;i<a.length;i++) switch(a[i]) {
			case "stacks":
				break;
			default:
				data[a[i]] = this[a[i]];
		}
		localStorage[this._prefix+this.id] = JSON.stringify(data);
	},
	load: function(callback) {
		var s = false;
		if (localStorage[this._prefix+this.id]) {
			var data = JSON.parse(localStorage["i_"+this.id]);
			if (data.lastLoad > this.lastLoad) {
				var a = Object.getOwnPropertyNames(data);
				for (var i=0;i<a.length;i++) switch(a[i]) {
					case "id":
						break;
					default:
						this[a[i]] = data[a[i]];
				}
			}
			if (this.tab && !data.tab) s = true;
		}
		if (!this.lastLoad) this.loadData(callback);
		else if (callback) {
			if (s) this.save();
			callback(this);
		}
	},
	get count() {
		var a = 0;
		for (var sId in this.stacks) a += this.stacks[sId].count;
		return a;
	},
	get value() {
		return this.mValue && this.mValue > this.sValue ? this.mValue : this.sValue;
	},
	set value(a) {
		this.sValue = a;
	},
	auction: function(options, callback) {
		/*
			repeat
			amount
			duration
				1:	1 day
				2:	3 days
				3:	7 days
			price
			currency
		*/
		var item = this;
		if (!options.repeat) options.repeat = 1;
		if (!options.amount) options.amount = 1;
		if (!options.duration) options.duration = 1;
		
		var data = {
			id: this.id,
			tab: this.tab,
			ver: "con",
			qty: options.amount,
			drtn: options.duration,
			cost: options.price,
		};
		switch(options.currency) {
			case "t":
			case "treasure":
				data.cur = 0;
				break;
			case "g":
			case "gem":
			case "gems":
				data.cur = 1;
				break;
			default:
				break;
		}
		var d = 0;
		var c = function() {
			var a,b,stack;
			for (var sId in item.stacks) {
				a = item.stacks[sId].count;
				if (a >= options.amount) {
					if (!stack || b > a - options.amount) {
						b = a - options.amount;
						stack = sId;
						if (b == 0) break;
					}
				}
			}
			if (d++ >= options.repeat || !stack) {
				if (callback) callback(item);
				return;
			}
			data.stack = stack;
			$.ajax({
				type: "POST",
				data: data,
				url: HOST+"/includes/ol/ah_sell.php",
				cache: false,
			}).done(function(html) {
				item.stacks[stack].count -= options.amount;
				setTimeout(c, 1000);
			});
		}
		c();
	},
};

/*
	food
	mats
	app
	fam
	battle
	skins
	other
*/
Hoard = {
	Page: {
		create: function(html, tab) {
			return Object.create(this).init(html, tab);
		},
		init: function(html, tab) {
			var doc = (new DOMParser()).parseFromString(html, "text/html");
			this.tab = tab;
			this.items = {};
			
			var $items = $(doc).find('a.clue').parent('span');
			var i, item, $item, a, itemId;
			for (i=0;i<$items.length;i++) {
				item = null;
				$item = $($items[i]);
				a = $item.find('a.clue').attr('rel').match(/\?id=([^&]+)&tab=([^&]+)/);
				itemId = Number(a[1]);
				if (this.items[itemId]) item = this.items[itemId];
				else {
					item = Item.extract($item.find('a.clue[rel]'), {tab: tab});
					this.items[itemId] = item;
				}
				item.addStack($item);
			}
			
			a = $($(doc).find("#invent").children("div")[0]);
			if (a.find("img[src='/images/layout/arrow_left.png']")) this.pageBefore = true;
			if (a.find("img[src='/images/layour/arrow_right.png']")) this.pageAfter = true;
			this.page = Number(a.find('span').text());
			if (this.pageAfter) {
				a = a.find('a[onclick]');
				this.maxPage = Number($(a[a.length-2]).text());
			} else this.maxPage = this.page;
			
			return this;
		},
	},
	getPage: function(data, callback)	 {
		$.ajax({
			data: data,
			url: HOST+"/includes/hoard_main.php",
		}).done(function(html) {
			var p = Hoard.Page.create(html, data.tab);
			if (callback) callback(p);
		});
	},
	getTab: function(tab, callback) {
		var a = this;
		var items = {};
		var pageCheck = [];
		var addItems = function(items2) {
			for (var sId in items2) {
				if (items[sId]) items[sId].join(items2[sId]);
				else items[sId] = items2[sId];
			}
		};
		var i = 1;
		var b = function(page) {
			if (page) {
				addItems(page.items);
				if (!pageCheck[page.maxPage]) pageCheck[page.maxPage] = null;
				pageCheck[page.page] = true;
			}
			do {
				if (!pageCheck[i]) {
					return setTimeout(function() {
						a.getPage({tab: tab, page: i}, b);
					}, 300);
				}
			} while (++i < pageCheck.length);
			if (callback) callback(items);
		};
		b();
	},
};

if (chrome.webRequest) {
	chrome.webRequest.onBeforeSendHeaders.addListener(function(details) {
		var mods = {};
		if (details.tabId) {
			var r = false;
			for (var i=0;i<details.requestHeaders.length;i++) {
				switch(details.requestHeaders[i].name) {
					case 'Referer':
						r = true;
						break;
				}
			}
			if (!r) {
				details.requestHeaders.push({
					name: "Referer",
					value: HOST+"/main.php",
				})
				mods.requestHeaders = details.requestHeaders;
			}
		}
		return mods;
	}, {
		'urls': [
			"*://*.flightrising.com/includes/itemajax.php*"
		]
	}, ['requestHeaders', 'blocking']);
}

$(function() {
	var itemsFound = 0;
	var $items = $('a.clue[rel]');
	for (var i=0;i<$items.length;i++) {
		var $item = $($items[i]);
		var a = $item.attr('rel').match(/\?id=([^&]+)&tab=([^&]+)/);
		if (a) {
			var item = Item.create(a[1], a[2], $item.find('img').attr('src'));
			itemsFound++;
		}
	}
	console.debug("Found "+itemsFound+" items!");
});