_a = 0;

c_xscript = new CrossScriptConnection("coli");

abilities = null;
xpTable = null;
coli_sockets = [];
xscript = new CrossScriptConnection("coliseum");
damageCalcs = {};

quickMode = false;
targetLevel = 0;
preLevel = false;
battles = 0;
deaths = 0;
venueXp = {};
autoFight = false;
stopClicked = false;

function BattleState(s, data) {
	this.socket = s;
	this.loadedIn = false;
	this.ended = false;
	
	this.id = data.battleId;
	this.bgm = data.bgm;
	this.firstTurn = data.firstTurn;
	this.currentTurn = 2;
	this.pveSet = data.pveSet;
	this.venue = data.venue;
	this.multiplayer = data.versusMode;
	
	this.characters = {};
	this.playerTeam = [];
	for (var i=0;i<data.playerSet.length;i++) {
		var a = Object.create(Character).load(data.playerSet[i]);
		a.battle = this;
		this.playerTeam.push(a);
		this.characters[a.id] = a;
	}
	this.enemyTeam = [];
	for (var i=0;i<data.enemySet.length;i++) {
		var a = Object.create(Character).load(data.enemySet[i]);
		a.battle = this;
		a.enemy = true;
		this.enemyTeam.push(a);
		this.characters[a.id] = a;
	}
	
	this.updateTurns(data.turnArray);
}
BattleState.prototype.useAbility = function(ability, target, targets) {
	var data = {
		ability: (typeof(ability) == 'object' ? ability.id : ability),
		abilityId: Math.floor(Math.random() * 1000),
		battleId: this.id,
		caster: {
			ai: this.turnOrder[0].ai,
			id: this.turnOrder[0].id,
		},
		aoeTargets: [],
		target: (target ? {
			ai: target.ai,
			id:	target.id,
		} : {}),
	}
	if (targets) {
		throw "Multi-targetting not supported!";
	}
	this.socket.emit('coliseum_useAbility', data);
};
BattleState.prototype.updateTurns = function(turnOrder) {
	// console.debug("Updating turn order...");
	if (turnOrder.currentTurn) this.currentTurn = turnOrder.currentTurn;
	if (turnOrder.array) turnOrder = turnOrder.array;
	var a = {};
	this.turnOrder = [];
	for (var i=0;i<turnOrder.length;i++) {
		var b = this.characters[turnOrder[i].id].load(turnOrder[i]);
		this.turnOrder.push(b);
	}
};
BattleState.prototype.makeMove = function() {
	if (this.ended || !autoFight) {
		this.moveQueued = true;
		return;
	}
	this.moveQueued = false;
	var caster = this.turnOrder[0];
	var enemies = [];
	var a = {};
	var j = 0;
	for (var i=1;i<this.turnOrder.length;i++) {
		var b = this.turnOrder[i];
		if (!b.enemy) j++;
		if (!a[b.id]) {
			a[b.id] = true;
			if (b.enemy && !b.dead) {
				b.turn = j;
				enemies.push(b);
			}
		}
	}
	for (var i in this.characters) if (!a[i]) {
		a[i] = true;
		var b = this.characters[i];
		if (b.enemy && !b.dead) {
			enemies.push(b);
			b.turn = null;
		}
	}
	if (enemies.length < 1) {
		console.error("Out of enemies! Lag?");
		//battle at same venue 
		console.log("Starting new battle at " + this.venue);
		this.socket.startBattle(this.venue);
		return;
	}
	
	// Filter usable abilities
	var skills = {};
	for (var i=0;i<caster.abilities.length;i++) {
		var b = abilities[caster.abilities[i]];
		if (b && b.breath <= caster.breath && b.slotType <= 2) {
			skills[b.name] = b;
		}
	}
	
	// Use defend if no usable abilities?
	var noAttack = false;
	for (var i in skills) {
		if (damageCalcs[i]) {
			noAttack = true;
			break;
		}
	}
	if (!noAttack) {
		var firstSkill = abilities[caster.abilities[0]];
		switch(firstSkill.name) {
			case "Meditate":
			case "Anticipate":
				this.useAbility(firstSkill, caster);
				return;
			default:
				this.useAbility(57, caster);
				return;
		}
	}
	
	// Determine ability and target
	var bestTarget = null;
	var bestSkill = null;
	var rating = 0;
	for (var i=0;i<enemies.length;i++) {
		var enemy = enemies[i];
		for (var j in skills) {
			var r = 10; // default value, is very high
			a = damageCalcs[j];
			if (!a) continue;
			var b = a(caster, enemy); // b is how much damage will be dealt to the enemy, based on calculations by var a
			var skill = skills[j];

			if (skill.breath <= 0) { // evaluate scratch
				if (b >= enemy.hp) {r = 1;} // scratch can kill enemies in one shot! highest priority to generate breath
				if (enemies.length == 2 && caster.breath < 70) {r = 3;}  // generate breath when fewer enemies
				else {r = 6;} 
			} 
			else if (skill.name == "Eliminate") { // evaluate eliminate
				if (b > enemy.hp) {
					r = 5; 
				} // if eliminate can one shot opponent, prioritize it
				else if (b < enemy.hp && enemy.hp / b > 2 && caster.breath > 70) {r = 4;} // facing boss and spare breath
				else {r = 10;} // stick with scratch lol
			}

			if (bestTarget == null || r < rating) {
				bestTarget = enemy;
				bestSkill = skill;
				rating = r;
			} //scratch's rating is r first, then when eliminate is evaluated the r value is compared to the old scratch rating 
		}
	}
	if (bestTarget != null) {
		// Send ability and target
		// console.groupCollapsed("Recommend move:");
		// console.log(bestTarget);
		// console.log(bestSkill);
		// console.log("Rating: "+rating);
		// console.groupEnd();
		return this.useAbility(bestSkill, bestTarget);
	}
	console.error("Unable to determine next move!");
};

var Character = {
	load: function(data) {
		this.id = data.id;
		this.realId = data.realid;	// null if dragon
		this.team = data.team;
		this.name = data.name;
		this.ai = data.ai;
		
		this.characteristics = data.characteristics;
		this.conditions = data.conditions;
		this.dead = data.dead;
		
		this.element = data.element;
		/*
			0	Neutral
			1	Earth
			2	Plague
			3	Wind
			4	Water
			5	Lightning
			6	Ice
			7	Shadow
			8	Light
			9	Arcane
			10	Nature
			11	Fire
		*/
		
		this.breed = data.breed;
		/*
			0	Unknown
			1	Fae
			2	Guardian
			3	Mirror
			4	Pearlcatcher
			5	Ridgeback
			6	Tundra
			7	Spiral
			8	Imperial
			9	Snapper
			10	Wildclaw
		
			12	Coatl
			13	Skydancer
		*/
		
		this.gender = data.gender;
		/*
			0	Male
			1	Female
			2	N/A
		*/
		
		this.owner = data.owner;
		this.dragonName = data.dragonname;
		this.xp = data.xp_now;
		
		this.hp = data.health;
		this.maxHp = data.maxHP;
		this.breath = data.breath;
		
		this.level = data.level;
		this.strength = data.strength;
		this.mind = data.mind;
		this.vitality = data.vitality;
		this.agility = data.agility;
		this.defense = data.defense;
		this.quickness = data.quickness;
		this.intellect = data.intellect;
		this.statpoints = data.statpoints;	// null if not dragon
		
		this.abilities = [];
		for (var i=0;true;i++) {
			var b = data["a"+i];
			if (b == null) {
				break;
			} else if (b != 0) {
				this.abilities[i] = b;
			}
		}
		return this;
	},
	_setStat: function(a, b) {
		this[a] = b;
	},
	_getStat: function(a, b) {
		var i = this[a];
		if (b && this.conditions) for (var j in this.conditions) if (this.conditions[j][b]) i += this.conditions[j][b];
		return i;
	},
	addStats: function(stats) {
		var data = {
			dragonid: this.id,
			strength: 0,
			defense: 0,
			agility: 0,
			vitality: 0,
			quickness: 0,
			intellect: 0,
			mind: 0,
		};
		for (var i in stats) {
			data[i] = stats[i];
		}
		this.battle.socket.emit("d290j1-dfaf", data);
	},
};

var Ability = {
	obf: [],
	load: function(data) {
		var obf = Ability.obf;
		this.realId = data[obf[0]];
		this.slotType = data[obf[1]];
		/*
			0	None (Item)
			1	Primary Attack
			2	Secondary Attack
			3	Augment
			4	Accessory
		*/
		this.id = data[obf[2]];
		this.name = data[obf[3]];
		this.description = data[obf[4]];
		this.value = Number.parseInt(data[obf[6]]);
		this.element = Number.parseInt(data[obf[9]]);
		this.allowedElement = {
			earth: Number.parseInt(data[obf[17]]),
			plague: Number.parseInt(data[obf[18]]),
			wind: Number.parseInt(data[obf[19]]),
			water: Number.parseInt(data[obf[20]]),
			lightning: Number.parseInt(data[obf[21]]),
			ice: Number.parseInt(data[obf[22]]),
			shadow: Number.parseInt(data[obf[23]]),
			light: Number.parseInt(data[obf[24]]),
			arcane: Number.parseInt(data[obf[25]]),
			nature: Number.parseInt(data[obf[26]]),
			fire: Number.parseInt(data[obf[27]]),
		};
		this.breath = data[obf[38]];
		return this;
	},
};

(function() {
	var formula0 = function(a, b, d) {
		return function(c, t) {
			var dmg = 4 * c[b] + a;
			var red = Math.floor(t[d] / 3) * 4 + (t[d] % 3);
			return dmg - red;
		};
	}
	var phys0 = function(a) {
		return formula0(a, 'strength', 'defense');
	};
	var spell0 = function(a) {
		return formula0(a, 'intellect', 'mind');
	};
	damageCalcs['Clobber'] = phys0(4);
	damageCalcs['Scratch'] = phys0(12);
	damageCalcs['Bite'] = phys0(12);
	damageCalcs['Shred'] = phys0(17);
	damageCalcs['Sap'] = phys0(15);
	damageCalcs['$Slash'] = phys0(75);
	damageCalcs['Eliminate'] = function(c, t) {
		var dmg = 12 * c.strength + 75;
		var red = 4 * t.defense;
		return dmg - red;
	};
	damageCalcs['Contuse'] = spell0(19);
	damageCalcs['$Bolts'] = spell0(75);
	damageCalcs['$Special'] = function(c, t) {
		var dmg = 6 * c.intellect + 17;
		var red = 2 * t.mind;
		return dmg - red;
	};
	damageCalcs['Enfeeble'] = function(c, t) {
		var dmg = 2 * c.intellect + 100;
		var red = Math.floor(t.mind / 3) * 2 + Math.round((t.mind % 3) / 3);
		return dmg - red;
	};

	function QStat(name, name2) {
		return {
			get: function() {return this._getStat(name, name2);},
			set: function(a) {this._setStat(name, a);},
		};
	}
	Object.defineProperties(Character, {
		"strength": QStat("_strength", "strength"),
		"intellect": QStat("_intellect", "intellect"),
		"defense": QStat("_defense", "defense"),
	});
})();

(function() {
	//console.debug("Injecting coliseum hooks...");
	
	function loadAbilities(a) {
		//console.debug("Searching for ability map...")
		abilities = {};
		var d = [];
		for (var b in window) {
			var c = a.indexOf(b+" =");
			if (c > -1) d[c] = b;
		}
		var vars = [];
		for (var i=0;i<d.length;i++) if (d[i] != undefined) vars.push(d[i]);
		for (var k=0;k<vars.length;k++) {
			var obf_abilities = window[vars[k]];
			if (typeof(obf_abilities) != "object" || !obf_abilities.push) continue;
			var b = a.match(new RegExp(vars[k]+"\\[\\d+\\] = {([^}]+)"));
			if (!b) continue;
			b = b[1].split(',\n');
			var e = [];
			for (var i=0;i<b.length;i++) {
				var c = b[i].trim();
				d = c.split(':');
				if (d[1]) e.push(d[0]);
			}
			Ability.obf = e;
			var j = 0;
			for (var i=0;i<obf_abilities.length;i++) {
				if (obf_abilities[i] != undefined) {
					b = Object.create(Ability).load(obf_abilities[i]);
					abilities[b.realId] = b;
					j++;
				}
			}
			//console.debug(j.toString()+" abilities extracted!");
			return true;
		}
		abilities = null;
	}
	
	(function() {
		//console.debug("Extracting coliseum data...");
		var a = $('#super-container').find('script[type="text/javascript"]');
		var d = new RegExp("var lastLevel = ([a-zA-Z0-9]+)\\[");
		for (var i=0;i<a.length;i++) {
			var c = $(a[i]).html();
			if (!abilities) {
				if (c.indexOf("function (o) ") > -1) {
					if (loadAbilities(c)) continue;
					console.error("Failed to load ability map!");
					continue;
				}
			}
			if (!xpTable) {
				var e = c.match(d);
				if (e) {
					xpTable = window[e[1]];
					continue;
				}
			}
		}
		if (abilities == null) console.error("Failed to find ability map!");
		if (xpTable == null) console.error("Failed to find xp table!");
	})();
	
	var A = {
		_returnToMenu: function(s) {
			// Wipe last battle
			s.battle = null;
		},
		onWinBattle: function(s, a) {
			// Battle won, process results
			var battle = s.battle;
			var z = battle.ended;
			if (!z) {
				var c = _a++;
				battle.z = c;
				battle.ended = true;
				a = a.args[0];
				battle.chain = a.chain;
				battle.achievements = a.chievs;
				battle.items = a.loot;
				addItems(battle.items);
				battle.xpGain = a.xp;
				var vXp = venueXp[battle.venue];
				if (!vXp || vXp < battle.xpGain) venueXp[battle.venue] = s.battle.xpGain;
				updateProgress(battle);
				setTimeout(function() {
					if (battle.z != c) return;
					s.startBattle(s.battle.venue);
				}, quickMode ? 500+1500*Math.random() : 5000+3000*Math.random());
			}
		},
		onLoseBattle: function(s) {
			// Battle lost
			var battle = s.battle;
			var z = battle.ended;
			if (!z) {
				var c = _a++;
				battle.z = c;
				battle.ended = true;	// Sit and wait for action
				deaths++;
				updateProgress(battle);
				if (autoFight) {
					setTimeout(function() {
						if (battle.z != c) return;
						s.startBattle(battle.venue);
					}, quickMode ? 1000+1000*Math.random() : 5000+3000*Math.random());
				}
			}
			A.feedAll();
		},
		
		_beginBattle: function(s) {
			// Prepare for new battle
			s.battle = null;
		},
		onFinalizeBattle: function(s, a) {
			// New battle received!
			s.battle = new BattleState(s, a.args[0]);
			battles++;
			updateProgress(s.battle);
		},
		onBattleLoaded: function(s) {
			// Notifcation that server verified we are ready for battle
			s.battle.loadedIn = true;
			
			// Check if we move first
			if (s.battle.turnOrder[0].team == 1) {
				// console.debug("Our turn!");
				setTimeout(function() {
					s.battle.makeMove();
				}, quickMode ? 250+750*Math.random() : 1500+(1500*Math.random()));
			} else if (s.battle.playerTeam.length < 1) {
				// Bug, we have no team so kill the battle!
				s.killBattle();
			}
		},
		onAbilityUse: function(s, a) {
			// Move made, update character stats
			a = a.args[0];
		
			// Apply changes in case modified characters don't move within 6 turns
			if (typeof(a.caster) == "object") {
				s.battle.characters[a.caster.id].load(a.caster);
			} else {
				s.battle.characters[a.caster].breath -= a.breath;
			}
			if (typeof(a.target) == "object") {
				s.battle.characters[a.target.id].load(a.target);
			} else {
				s.battle.characters[a.target].hp -= a.damage;
			}
			// var critical = a.critical;
			/*
			0	None
			1	Critical
			2	Dodge!
			3	Miss!
			*/
			// var reflect = a.reflect;	// ?
			// var i = a.id;	// ?
			// var clash = a.clash;
		
			if (a.newTurns) {
				// Contains final character stats
				s.battle.updateTurns(a.newTurns);
				
				// Check if we move
				if (s.battle.turnOrder[0].team == 1) {
					// console.debug("Our turn!");
					setTimeout(function() {
						s.battle.makeMove();
					}, quickMode ? 250+500*Math.random() : 1000+(1000*Math.random()));
					return;
				}
				if (s.battle.playerTeam.length < 1) {
					// False battle, kill client
					s.disconnect();
					// location.reload();
				}
			}
		},
		onCharDeath: function(s, a) {
			// Must be done since dead characters never get another turn to update their status
			for (var i=0;i<a.args.length;i++) {
				var character = s.battle.characters[a.args[i]];
				if (character) {
					// console.debug(character.name+" is dead!");
					character.dead = true;
				} else console.error("Unable to locate: "+a[i]);
			}
		},
		onOrganizeParty: function(s, a) {
			var chars = a.args[0];
			s.dragons = {};
			for (var i=0;i<chars.length;i++) {
				var b = chars[i];
				var a = s.dragons[b.id];
				if (a == null) {
					a = Object.create(Character).load(b);
					s.dragons[a.id] = a;
				} else {
					a.load(b);
				}
			}
		},
		onPartyAuthorized: function(s) {
			if (this.setPartyCallback) {
				var a = this.setPartyCallback;
				this.setPartyCallback = null;
				a(true);
			}
		},
		setParty: function(dragonIds, callback) {
			this.setPartyCallback = callback;
			this.emit("coliseum_authorizeParty", dragonIds);
		},
		startBattle: function(venue, battleId) {
			if (stopClicked || !autoFight) {
				resetStopBtn();
				return;
			}
			var battle = this.battle;
			updateProgress(battle);
			if (battle != null && battle.ended) {
				if (targetLevel) {
					var playerTeam = battle.playerTeam;
					var xpGain = battle.xpGain ? battle.xpGain : 0;
					if (preLevel) {
						var vXp = venueXp[venue] ? venueXp[venue] : 1500;
						for (var i=0;i<playerTeam.length;i++) {
							var dragon = playerTeam[i];
							if (dragon.level < targetLevel && vXp + xpGain + dragon.xp >= xpTable[targetLevel-2]) {
								c_xscript(null, "highlight");
								return;
							}
						}
					} else {
						var keepGoing = false;
						for (var i=0;i<playerTeam.length;i++) {
							var dragon = playerTeam[i];
							if (dragon.level < targetLevel && xpGain + dragon.xp < xpTable[targetLevel-2]) {
								keepGoing = true;
							}
						}
						if (!keepGoing) {
							c_xscript(null, "highlight");
							return;
						}
					}
				}
			}
			var data = {};
			venue != undefined ? (data.venue = venue) : null, battleId != undefined ? (data.battleId = battleId) : null;
			this.emit("coliseum_beginBattle", data);
		},
		feedAll: function() {
			// console.debug("Feeding dragons");
			var xhr = new XMLHttpRequest();
			xhr.open("POST", "/includes/ol/feed.php", true);
			xhr.send("");
			return xhr;
		},
	}
	
	for (var i=0;i<sockets.length;i++) if (sockets[i].socket.options.resource.match(/^coliseum[0-9]*\/socket.io$/)) {
		var s = sockets[i];
		coli_sockets.push(s);
		s.dontEchoReceive("playerCount");
		s.addSendHandler("coliseum_beginBattle", "helper", A._beginBattle);
		s.addReceiveHandler("finalizeBattle", "helper", A.onFinalizeBattle);
		s.addReceiveHandler("loadedIn", "helper", A.onBattleLoaded);
		s.addReceiveHandler("finalizeAbility", "helper", A.onAbilityUse);
		s.addReceiveHandler("death", "helper", A.onCharDeath);
		s.addReceiveHandler("winBattle", "helper", A.onWinBattle);
		s.addReceiveHandler("loseBattle", "helper", A.onLoseBattle);
		s.addSendHandler("coliseum_returnToMenu", "helper", A._returnToMenu);
		s.addReceiveHandler("organize", "helper", A.onOrganizeParty);
		s.addReceiveHandler("partyAuthorized", "helper", A.onPartyAuthorized);
		s.startBattle = A.startBattle;
		s.feedAll = A.feedAll;
		s.setParty = A.setParty;
		// s.silentEcho = true;
	}
	
	(function() {
		var a = $("#volumeControl").parent();
		a.attr("style", a.attr("style").replace(/width:[^;]+;/, "").replace(/height:[^;]+;/, "margin-top: 10px;"));
		var b = a.parent();
		b.attr("style", a.attr("style").replace(/position:[^;]+;/, "margin-top: 10px;"));
		var main = $("<div style='font-size: 12px; padding-top: 5px; padding-right: 10px; padding-left: 10px;'></div>").appendTo(a);
		var autoFightBtn = $("<button style='width: 160px;'></button>").text("Auto-Fight").appendTo(main).click(function(evt) {
			autoFight = !autoFight;
			updateAutoFightBtn();
			evt.preventDefault();
			return false;
		});
		updateAutoFightBtn = function() {
			if (!autoFight) {
				autoFightBtn.text("Auto-Fight (Disabled)");
			} else {
				autoFightBtn.text("Auto-Fight (Enabled)");
			}
			if (coli_sockets[0].battle && coli_sockets[0].battle.moveQueued) {
				coli_sockets[0].battle.makeMove();
			}
		}
		main.append($("<br>"));
		var quickButton = $('<button style="width: 160px; margin-top: 5px;">Fast Mode</button>').click(function(evt) {
			quickMode = !quickMode;
			updateQuickButton();
		}).appendTo(main);
		updateQuickButton = function() {
			quickButton.text("Fast Mode ("+(quickMode ? "Enabled" : "Disabled")+")");
		};
		main.append($('<br>'));
		var stopBtn = $("<button style='width: 160px; margin-top: 5px;'></button>").text("Stop").appendTo(main).click(function(evt) {
			stopBtn.text("Stopping").attr("disabled", true);
			stopClicked = true;
			evt.preventDefault();
			return false;
		});
		resetStopBtn = function() {
			stopBtn.text("Stop").attr("disabled", false);
			stopClicked = false;
		}
		main.append($("<br>"));
		var lastState;
		b = $("<span>Target Level: </span>").appendTo(main);
		var tLField = $("<input type='text' style='width: 2em; margin-top: 5px;'></input>").val('0').appendTo(b).change(function(evt) {
			var i = Number(tLField.val());
			targetLevel = i != NaN ? i : 0;
			if (targetLevel) {
				autoFight = true;
				updateAutoFightBtn();
			}
			updateProgress();
		});
		b.append("<div id='progress' style='position: relative; display: inline-block; left: 0.5em; width: 3em; text-align: right'></div>");
		updateTitle = function(a) {
			var title = "Coli";
			if (a) {
				title += " | "+a;
			}
			if (battles) {
				title += " | "+battles;
				if (deaths) {
					title += "/"+deaths;
				}
			}
			document.title = title;
		};
		updateProgress = function(a) {
			if (a) lastState = a;
			if (!targetLevel) {
				updateTitle();
				$('#progress').text('');
				return;
			}
			if (!lastState) {
				updateTitle();
				$('#progress').text("N/A");
				return;
			}
			var lowest = null;
			var playerTeam = lastState.playerTeam;
			var xpGain = lastState.xpGain ? lastState.xpGain : 0;
			var targetXp = xpTable[targetLevel-2];
			if (preLevel) {
				var vXp = venueXp[venue] ? venueXp[venue] : 1500;
				for (var i=0;i<playerTeam.length;i++) {
					var dragon = playerTeam[i];
					if (dragon.level >= targetLevel) continue;
					a = (vXp + xpGain + dragon.xp) / targetXp;
					if (lowest == null || a > lowest) lowest = a;
				}
				if (lowest == null) lowest = 1;
			} else {
				for (var i=0;i<playerTeam.length;i++) {
					var dragon = playerTeam[i];
					if (dragon.level >= targetLevel || xpGain + dragon.xp >= targetXp) continue;
					a = (xpGain + dragon.xp) / targetXp;
					if (lowest == null || a < lowest) lowest = a;
				}
				if (lowest == null) lowest = 1;
			}
			if (lowest >= 1) {
				lowest = "Done!";
			} else {
				lowest = (lowest * 100).toFixed(0)+"%";
			}
			updateTitle(lowest);
			$('#progress').text(lowest);
		}
		main.append($("<br>"));
		
		loot = {};
		var Item = {
			init: function(a) {
				this.description = a.desc;
				this.id = a.id;
				this.name = a.name;
				this.rarity = a.rarity;
				this.subtype = a.subtype;
				this.value = a.value;
				this.count = 0;
				
				this.ele = $("<div style='display: inline-block; margin: 5px;'></div>");
				this.elePic = $("<a></a>").append($("<img>").attr("src", "/images/cms/"+this.subtype+"/"+this.id+".png")).appendTo(this.ele).attr("rel", "/includes/itemajax.php?id="+this.id+"&tab="+this.subtype).attr("class", "clue").cluetip({height: 'auto'});
				this.eleText = $("<span style='position: relative; float: right;/* top: -14px; background-color: white;*/ padding-right: 7px;/* padding-bottom: 7px; padding-top: 2px; padding-left: 2px;*/'></span>").appendTo(this.ele.append("<br>"));
				
				this.update();
				
				return this;
			},
			update: function() {
				this.eleText.text("x"+this.count);
			},
		};
		var lootTable = $("<div style='margin-top: 20px;'><h3>Loot</h3><hr width=100%></div>").appendTo(main);
		addItems = function(iRaw) {
			if (iRaw) for (var i=0;i<iRaw.length;i++) {
				var a = iRaw[i];
				var b = null;
				if (loot[a.id]) {
					b = loot[a.id];
				} else {
					b = Object.create(Item).init(a);
					loot[b.id] = b;
					lootTable.append(b.ele);
				}
				b.count++;
				b.update();
			}
		};
		
	})();
	
	//console.debug("Coliseum hooks injected!");
})();
